/*
 * SourceReader.java
 *
 * Created on 6  2002 ., 14:02
 */

package com.fn.spcave.compiler;


import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.Reader;


/**
 *
 * @author  asso
 */
public class ChainedSourceReader implements SourceReader {
    
    private int line;
    private Reader reader;
    private int[] stack;
    private int stackLen;
    private String name;
        
    private static final int STACK_SIZE = 20;
    private static final int STACK_INCR = 10;    
    
    public ChainedSourceReader(String name, int firstLineNo, Reader reader) throws ScException {
        this.reader = reader;
        this.name = name;
        line = firstLineNo;
        stackLen = 0;
        stack = new int[STACK_SIZE];
    }
    
    /** Creates a new instance of SourceReader */
    public ChainedSourceReader(File file, Locale locale) throws ScException {
        try {
            if (locale != null)
                reader = new InputStreamReader(new FileInputStream(file), locale.getEncoding());
            else
                reader = new InputStreamReader(new FileInputStream(file));
        } catch (Exception e) {
            throw new ScException("Error opening template file " + file.getPath());
        }
        name = file.getPath();
        line = 1;
        stackLen = 0;
        stack = new int[STACK_SIZE];
    }
    
    private void updatePos(int ch) {
        if (ch == '\n')
            line++;
    }
    
    private void unupdatePos(int ch) {
        if (ch == '\n')
            line--;
    }
    
    public int getChar() throws ScException {
        int ch;
        if (stackLen > 0) {
            stackLen--;
            ch = stack[stackLen];
        } else {
            try {
                ch = (char)reader.read();
                if (ch == 65535)
                    ch = -1;
            } catch (IOException e) {
                throw new ScException("Error reading character: " + e.getMessage());
            }
        }
        if (ch != -1)
            updatePos(ch);
        return ch;
    }
    
    public void ungetChar(int ch) {
        if (stackLen >= stack.length) {
            int[] newStack = new int[stack.length + STACK_INCR];
            System.arraycopy(stack, 0, newStack, 0, stack.length);
            stack = newStack;
        }
        unupdatePos(ch);
        stack[stackLen] = ch;
        stackLen++;
    }
    
    public int getLineNumber() {
        return line;
    }
    
    public String getName() {
        return name;
    }
    
}
