/*
 * NewsFile.java
 *
 * Created on 22  2002 ., 11:53
 */

package com.fn.station;


import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import javax.xml.parsers.DocumentBuilder; 
import javax.xml.parsers.DocumentBuilderFactory;  
import javax.xml.parsers.FactoryConfigurationError;  
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;  
import org.xml.sax.SAXParseException;
import org.w3c.dom.Document;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import com.fn.spcave.compiler.*;


/**
 *
 * @author  asso
 */
public class NewsFile {
    
    private List news;
    
    public class NewsEntry {
        public String title;
        public String descr;
        public String body;
        public int day;
        public int month;
        public int year;
        
        public NewsEntry(Element root) throws ScException {
            day = XMLUtils.getTagAttrib(root, "day", 1);
            month = XMLUtils.getTagAttrib(root, "month", 1);
            year = XMLUtils.getTagAttrib(root, "year", 2002);
            NodeList nodeList = root.getChildNodes();
            int len = nodeList.getLength();
            int printedNewsCount = 0;
            for (int i = 0; i < len; i++) {
                Node node = nodeList.item(i);
                switch (node.getNodeType()) {
                    case Node.ELEMENT_NODE:
                        processElement((Element)node);
                        break;            
                    case Node.TEXT_NODE:
                        if (XMLUtils.isBlankText(node.getNodeValue()))
                            break;                    
                    default:
                        throw new ScException("Error in news file: unexpected element");
                }
            }
        }
        
        private void processElement(Element element) throws ScException {
            String name = element.getNodeName();
            if (name.equals("title"))
                title = XMLUtils.getTrimmedData(element);
            else if (name.equals("descr"))
                descr = XMLUtils.getTrimmedData(element);
            else if (name.equals("body"))
                body = XMLUtils.getTrimmedData(element);
        }
    }
    
    /** Creates a new instance of NewsFile */
    public NewsFile(File f) throws ScException {
        news = new ArrayList();
        
        Document document;
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            factory.setIgnoringComments(true);
            factory.setIgnoringElementContentWhitespace(true);
            DocumentBuilder builder = factory.newDocumentBuilder();
            document = builder.parse(f);
        } catch (SAXException sxe) {
            Exception x = sxe;
            if (sxe.getException() != null)
                x = sxe.getException();
            x.printStackTrace();
            throw new ScException("Error generated during parsing: " + x.getMessage());
        } catch (ParserConfigurationException pce) {
            pce.printStackTrace();
            throw new ScException("Parser with specified options can't be built");
        } catch (IOException ioe) {
            ioe.printStackTrace();
            throw new ScException("Error reading news '" + f.getPath() + "'");
        } catch (Exception e) {
            throw new ScException("Unknown exception in config parser: " + e.getMessage());
        }
        Element root = document.getDocumentElement();
        
        NodeList nodeList = root.getChildNodes();
        int len = nodeList.getLength();
        int printedNewsCount = 0;
        for (int i = 0; i < len; i++) {
            Node node = nodeList.item(i);
            switch (node.getNodeType()) {
                case Node.ELEMENT_NODE:
                    NewsEntry ne = new NewsEntry((Element)node);
                    news.add(ne);
                    break;            
                case Node.TEXT_NODE:
                    if (XMLUtils.isBlankText(node.getNodeValue()))
                        break;                    
                default:
                    throw new ScException("Error in news file: unexpected element");
            }
        }
    }
    
    Iterator getIterator() {
        return news.iterator();
    }
}
